<?php

/**
 * @file
 */

/**
 *
 */
function auctions_bids_form($form, &$form_state, $auction, $view_mode, $langcode) {
  global $user;
  $controller = auctions_get_controller();

  $form_state['auction']['auction'] = $auction;
  $form_state['auction']['minimum'] = auctions_get_minimum_bid_value($auction);
  $form_state['auction']['current_price'] = auctions_get_current_price($auction);
  $form_state['auction']['currency'] = auctions_get_settings('currency');

  if ($controller->isBiddingActive($auction)) {
    $form['current_price'] = array(
      '#type' => 'item',
      '#title' => t('Current price'),
      '#markup' => auctions_format_price($form_state['auction']['current_price'] * 100),
      '#field_suffix' => '<small>'.$form_state['auction']['currency'].'</small>',
    );

    if ($controller->userIsWinning($auction, $user->uid)) {
      $highest_bid = $controller->getHighestBidAmount($auction);
      $title = t('Your new bid');
      $default_value = $highest_bid + $controller->getBidStep($auction);
      $description = t('Your last bid is @amount', array('@amount' => $highest_bid));
    }
    else {
      $title = t('Your bid');
      $default_value = $form_state['auction']['minimum'];
      $description = '';
    }
    $form['bid_amount'] = array(
      '#type' => 'textfield',
      '#title' => $title,
      '#default_value' => $default_value,
      '#required' => TRUE,
      '#field_suffix' => $form_state['auction']['currency'],
      '#size' => 10,
      '#element_validate' => array('auctions_bids_bid_amount_validate'),
      '#description' => $description,
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Place a bid'),
      '#submit' => array('auctions_bids_place_a_bid_submit'),
    );
  }

  if ($controller->isInstantBuyActive($auction)) {
    $form_state['auction']['instant_buy_price'] = $controller->getInstantBuyPrice($auction);

    $form['price'] = array(
      '#type' => 'item',
      '#title' => t('Instant buy price'),
      '#markup' => auctions_format_price($controller->getInstantBuyPrice($auction), $auction->currency_code),
    );
    $form['instant_buy_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Buy now'),
      '#validate' => array('auctions_bids_instant_buy_validate'),
    );

    return $form;
  }

  return $form;
}

/**
 *
 */
function auctions_bids_bid_amount_validate($element, &$form_state, $form) {
  global $user;
  $controller = auctions_get_controller();
  $currency = '<small>' . $form_state['auction']['currency'] . '</small>';
  if ($form_state['values']['op'] == t('Place a bid')) {
    // We load auction again to make sure it' it's active.
    $auction = $controller->changeStatusIfNeeded($form_state['auction']['auction']);
    if ($auction->status != AUCTION_STATUS_ACTIVE) {
      form_error($element, t('Auction has ended.'));
    }

    // If user has entered bad data (letters, special char) is an error.
    $bid_amount = $form_state['values']['bid_amount'];
    if (!is_numeric($bid_amount)) {
      form_error($element, t('New bid is not a known price format.'));
    }

    // If user is bidding against self.
    $isWInning = $controller->userIsWinning($form_state['auction']['auction'], $user->uid);
    if ($isWInning) {
      form_error($element, t('You are tring to outbid yourself!'));
    }

    // If value is lower than auction min.
    $current_minimum = auctions_get_minimum_bid_value($form_state['auction']['auction']);
    $highest_bid_amount = $controller->getHighestBidAmount($form_state['auction']['auction']);
    $new_bid = floatval($bid_amount);
    if (($new_bid * 100) <= ($highest_bid_amount * 100)) {
      form_error($element, t('Your bid amount is too low. It has to be at least @minimum.',
        array('@minimum' => $highest_bid_amount . ' ' . $currency)));
    }

  }
}

/**
 *
 */
function auctions_bids_place_a_bid_submit($form, &$form_state) {
  global $user;
  $controller = auctions_get_controller();
  $bid_id = NULL;
  if ($controller->userIsWinning($form_state['auction']['auction'], $user->uid)) {
    $bid_id = $controller->getHighestBid($form_state['auction']['auction'])->bid_id;
  }

  $controller->saveBid(
    $form_state['auction']['auction'],
    $form_state['values']['bid_amount'],
    $form_state['auction']['currency'],
    $user->uid,
    AUCTION_BID_TYPE_STANDARD,
    $bid_id
  );
  drupal_set_message(t('Your bid has been saved.'));

}

/**
 *
 */
function auctions_bids_instant_buy_validate($form, &$form_state) {
  global $user;
  $controller = auctions_get_controller();

  // We load auction again to make sure it' it's active.
  $auction = $controller->changeStatusIfNeeded($form_state['auction']['auction']);

  if ($auction->status == AUCTION_STATUS_ACTIVE) {
    $controller->saveBid(
      $auction,
      $form_state['auction']['instant_buy_price'] / 100,
      $form_state['auction']['currency'],
      $user->uid,
      AUCTION_BID_TYPE_INSTANT_BUY,
      NULL
    );
    $controller->deactivate($auction);
  }
  else {
    form_set_error('', t('Auction has ended.'));
  }
}

/**
 * Return links to bidders profiles.
 *
 * @param array $variables
 *
 * @return string
 */
function theme_auctions_bids_overview($variables) {
  $output = '';
  foreach ($variables['bids'] as $bid) {
    $output .= auctions_bids_format_bid_overview_row($bid) . "\n";
  }
  return $output;
}

/**
 * Returns human-readable information about single bid.
 *
 * @param object $bid
 *
 * @return string
 */
function auctions_bids_format_bid_overview_row(&$bid) {
  $name = db_select('users', 'u')
    ->fields('u', array('name'))
    ->condition('uid', $bid->uid)
    ->execute()
    ->fetchColumn();

  $output = auctions_format_price($bid->amount, $bid->currency_code) . ' - '
          . l($name, 'user/' . $bid->uid) . ' - ';

  $output .= $bid->type == AUCTION_BID_TYPE_STANDARD ? 'standard bid' : 'instant buy';
  return $output;
}
